jQuery(document).ready(function ($) {
    var $textarea = $('textarea', 'div.submit-post');

    autosize($textarea);

    $textarea.focus(function (event) {
        var $submitPost = $(this).parents('div.submit-post');

        $submitPost.find('div.submit-post-actions').show();

        if ($submitPost.find('div.submit-post-buttons a').length) {
            $submitPost.find('div.submit-post-buttons').show();
        }

        var $form = $textarea.parents('form:first');
        var captcha = $form.data('captcha');

        if ('' != captcha && 0 === $submitPost.find('div#post_captcha').length) {
            $(captcha).insertBefore($form.find('div.submit-post-actions a[data-action="submit"]'));

            var items = $form.find('.g-recaptcha'), item, options;

            for (var i = 0, l = items.length; i < l; i++) {
                item = items[i];
                options = item.dataset ? item.dataset : {
                    sitekey: item.getAttribute('data-sitekey'),
                    theme:   item.getAttribute('data-theme'),
                    size:    item.getAttribute('data-size')
                };
                grecaptcha.render(item, options);
            }
        }
    });

    $('a[data-action="cancel"]', 'div.submit-post-actions').click(function (event) {
        event.preventDefault();

        var $submitPost = $(this).parents('div.submit-post');

        $submitPost.trigger('reset');
    });

    $('a[data-action="submit"]', 'div.submit-post-actions').click(function (event) {
        event.preventDefault();

        var $button = $(this);

        if ($button.isLoading()) {
            return false;
        }

        var $form     = $button.parents('form:first');
        var $textarea = $form.find('textarea:first');
        var data      = $form.serialize();

        if ('' === $.trim($textarea.val())) {
            $textarea.focus();
            return false;
        }

        $button.startLoading();

        $.post($form.attr('action'), data, function (response) {
            if (true === $.checkAjaxRedirect(response)) { return false; }

            $button.stopLoading();

            if (response.success) {
                $('div.posts').trigger('refresh');

                $button.parents('div.submit-post:first').trigger('reset');
            }
            else {
                $button.showTooltip(response);
            }
        }, 'json');
    });

    $(document).on('reset', 'div.submit-post', function () {
        var $element = $(this),
            $textarea = $element.find('textarea');

        $textarea.val('');
        $element.find('div.submit-post-actions').hide();

        if ($element.find('div.submit-post-buttons a').length) {
            $element.find('div.submit-post-buttons').hide();
        }

        var evt = document.createEvent('Event');
        evt.initEvent('autosize:update', true, false);
        $textarea[0].dispatchEvent(evt);

        $element.find('div.submit-post-media').html('');

        $element.find('div#post_captcha').parents('div.control-group:first').remove();
    });

    $(document).on('click', 'a[data-post-media="remove"]', function (event) {
        event.preventDefault();

        $(this).parents('div.template-media:first').remove();
    });

    Handlebars.registerPartial('mediaRemoveButton',
        '<a href="#" class="pull-right muted remove-media" data-post-media="remove">' +
        '<span class="fa fa-fw fa-times"></span>remove' +
        '</a>'
    );

    var $buttons = $('div.submit-post-buttons');
    var config   = $buttons.data('config');
    var source   = $('#media-button-template').html();
    var template = Handlebars.compile(source);

    $('script[data-media]').each(function (index, element) {
        var $element = $(element);
        var button = $element.data('config').button;
        var html = template({
            type: $element.data('media'),
            tooltip: button.tooltip,
            icon: button.icon
        });

        $buttons.append(html);
    });

    $(document).on('click', 'div.submit-post-buttons a[data-media-button]', function (event) {
        event.preventDefault();

        var type = $(this).data('media-button');

        if (!limitCheck(type)) {
            return false;
        }

        $buttons.trigger('media_button_' + type);
    });

    var counter = 0;

    $buttons.bind('insert', function (event, template, callback, limitCheckType) {
        if (limitCheckType && !limitCheck(limitCheckType)) {
            return false;
        }

        var $html = $(template({ id: counter++ }));

        $('div.submit-post-media').prepend($html);

        if (callback) {
            callback.call($html);
        }
    });

    function limitCheck(type) {
        var config = $('script[data-media="' + type + '"]').data('config');
        var limit = config.limit;

        if (limit <= $('div[data-media="' + type + '"]').length) {
            return false;
        }

        return true;
    }
});
